"""
$Id: PSCImprovementProposalFolder.py 24400 2006-06-04 22:38:43Z optilude $
"""

from AccessControl import ClassSecurityInfo

from Products.CMFCore import permissions

from Products.Archetypes.atapi import *
from Products.Archetypes import transaction
from Products.ATContentTypes.content.base import ATCTMixin

from Products.PloneSoftwareCenter.config import PROJECTNAME, IMPROVEMENTS_ID

from Products.CMFCore.utils import getToolByName

PSCImprovementProposalFolderSchema = OrderedBaseFolderSchema.copy() + Schema((

    StringField(
        name='id',
        required=0,
        searchable=1,
        mode='r', # Leave the custom auto-generated ID
        widget=StringWidget (
            label="Short name",
            description="Short name of the container - this should be 'roadmap' to comply with the standards.",
            i18n_domain="plonesoftwarecenter",
        ),
    ),

    StringField(
        name='title',
        default='Roadmap',
        searchable=1,
        accessor="Title",
        widget=StringWidget(
            label="Title",
            description="Enter a title for the container",
            i18n_domain="plonesoftwarecenter",
        ),
    ),

    StringField(
        name='description',
        default='Improvement proposals which will be considered for this project',
        searchable=1,
        accessor="Description",
        widget=TextAreaWidget(
            label="Description",
            description="Enter a description of the container",
            i18n_domain="plonesoftwarecenter",
        ),
    ),
    
    LinesField(
        name='proposalTypes',
        multiValued=1,
        required=1,
        default=['User interface', 'Architecture'],
        widget=LinesWidget(
            label='Roadmap proposal types',
            label_msgid="label_roadmap_types",
            description='You will have a roadmap available in your project, and you can add categories of enhancement specifications below.',
            description_msgid="help_roadmap_types",
            i18n_domain="plonesoftwarecenter",
        ),
    ),

))

class PSCImprovementProposalFolder(ATCTMixin, BaseBTreeFolder):
    """A proposal container has proposals, and a view for the listing."""

    __implements__ = (ATCTMixin.__implements__,
                      BaseBTreeFolder.__implements__,)

    content_icon = 'improvementproposal_icon.gif'
    archetype_name = 'Roadmap Section'
    immediate_view = default_view = 'psc_roadmap'
    suppl_views = ()
    schema = PSCImprovementProposalFolderSchema

    global_allow = False
    filter_content_types = True
    allowed_content_types = ('PSCImprovementProposal',)
    _at_rename_after_creation = True

    security = ClassSecurityInfo()

    typeDescMsgId = 'description_edit_improvementproposalcontainer'
    typeDescription = ('A roadmap section contains improvement '
                       'proposals for the project.')

    def _renameAfterCreation(self, check_auto_id=False):
        parent = self.aq_inner.aq_parent
        if IMPROVEMENTS_ID not in parent.objectIds():            
            # Can't rename without a subtransaction commit when using
            # portal_factory!
            transaction.savepoint(optimistic=True)
            self.setId(IMPROVEMENTS_ID)
        
    security.declareProtected(permissions.View, 'getStateTitle')
    def getStateTitle(self, state):
        """Given the state name, look up its title in portal_workflow
        """
        wftool = getToolByName(self, 'portal_workflow')
        wf = wftool.psc_improvementproposal_workflow
        return wf.states[state].title
        
    security.declareProtected(permissions.View, 
                                'getImprovementProposals')
    def getImprovementProposals(self, review_state = []):
        """Get the improvement proposals in this folder, sorted on plip number,
        as a list of catalog brains.
        """
        catalog = getToolByName(self, 'portal_catalog')
        
        query = {}
        query['portal_type'] = 'PSCImprovementProposal'
        query['path'] = '/'.join(self.getPhysicalPath())
        
        if review_state:
            query['review_state'] = review_state
        
        proposals = catalog.portal_catalog(query)
                                    
        def sortProposals(x, y):
            try: xval = int(x.getId)
            except ValueError: xval = 0
            try: yval = int(y.getId)
            except ValueError: yval = 0
            return cmp(xval, yval)
                                    
        proposals = [p for p in proposals]
        proposals.sort(sortProposals)
        return proposals

registerType(PSCImprovementProposalFolder, PROJECTNAME)
