"""
$Id: PloneSoftwareCenter.py 24613 2006-06-08 23:40:22Z optilude $
"""

from AccessControl import ClassSecurityInfo

from Products.CMFCore import permissions
from Products.CMFCore.utils import getToolByName

from Products.Archetypes.atapi import *
from Products.ATContentTypes.content.base import ATCTMixin, updateActions
from Products.ArchAddOn.Fields import SimpleDataGridField
from Products.ArchAddOn.Widgets import SimpleDataGridWidget


from Products.PloneSoftwareCenter.config import PROJECTNAME

PloneSoftwareCenterSchema = OrderedBaseFolder.schema.copy() + Schema((

    TextField('description',
        searchable=1,
        accessor='Description',
        storage=MetadataStorage(),
        widget=TextAreaWidget(
            label_msgid='label_package_area',
            label='Description',
            description_msgid='help_package_area',
            description='Description for the software center.',
            i18n_domain='plonesoftwarecenter',
            rows=6,
        ),
    ),

    SimpleDataGridField('availableCategories',
        columns=3,
        column_names=('id', 'title', 'description'),
        default=[
            'standalone|Stand-alone products|Projects that are self-contained.', 
            'add-on|Add-on components|Projects that provide additional functionality.', 
            'infrastructure|Infrastructure|Projects that provide services.',
        ],
        widget=SimpleDataGridWidget(
            label='Categories',
            label_msgid='label_categories_vocab',
            description='Define the available categories for classifying the projects. The format is Short Name | Long name | Description. The id must be unique.',
            description_msgid='help_categories_vocab',
            i18n_domain='plonesoftwarecenter',
            rows=6,
        ),
    ),

    SimpleDataGridField('availableLicenses',
        column_names=('id', 'title', 'url'),
        columns=3,
        default=[
            'GPL|GPL - GNU General Public License|http://creativecommons.org/licenses/GPL/2.0/',
            'LGPL|LGPL - GNU Lesser General Public License|http://creativecommons.org/licenses/LGPL/2.1/',
            'BSD|BSD License (revised)|http://opensource.org/licenses/bsd-license',
            'Freeware|Freeware|http://wikipedia.org/Freeware',
            'Public Domain|Public Domain|http://creativecommons.org/licenses/publicdomain',
            'OSI|Other OSI Approved|http://opensource.org/licenses',
            'ZPL|Zope Public License (ZPL)|http://opensource.org/licenses/zpl',
            'Commercial|Commercial License|http://plone.org/documentation/faq/plone-license',
        ],
        widget=SimpleDataGridWidget(
            label='Available licenses',
            label_msgid='label_licenses_vocab',
            description='Define the available licenses for software releases. The format is Short Name | Title | URL.',
            description_msgid='help_licenses_vocab',
            i18n_domain='plonesoftwarecenter',
            rows=6,
        ),
    ),

    LinesField('availableVersions',
        default=[
            'Plone 2.1',
            'Plone 2.0.5',
            'Plone 2.0',
            'Plone 1.0',
        ],
        widget=LinesWidget(
            label='Versions',
            label_msgid='label_versions_vocab',
            description='Define the vocabulary for versions that software releases can be listed as being compatible with.',
            description_msgid='help_versions_vocab',
            i18n_domain='plonesoftwarecenter',
            rows=6,
        ),
    ),

    LinesField('availablePlatforms',
        default=[
            'All platforms',
            'Linux',
            'Mac OS X',
            'Windows',
            'BSD',
            'UNIX (other)'
        ],
        widget=LinesWidget(
            label='Platforms',
            label_msgid='label_platforms_vocab',
            description='Define the available platforms for software files.',
            description_msgid='help_platforms_vocab',
            i18n_domain='plonesoftwarecenter',
            rows=6,
        ),
    ),

))


class PloneSoftwareCenter(ATCTMixin, BaseBTreeFolder):
    """A simple folderish archetype for the Software Center."""

    __implements__ = (ATCTMixin.__implements__,
                      BaseBTreeFolder.__implements__)

    content_icon = 'product_icon.gif'

    archetype_name = 'Software Center'
    metatype = 'PloneSoftwareCenter'
    immediate_view = default_view = 'plonesoftwarecenter_view'
    suppl_views = ()

    global_allow = 1
    filter_content_types = 1
    allowed_content_types = ('PSCProject',)
    schema = PloneSoftwareCenterSchema
    _at_rename_after_creation = True

    security = ClassSecurityInfo()
    
    typeDescription = "A container for software releases"

    actions = updateActions(ATCTMixin,
        ({
        'id'          : 'local_roles',
        'name'        : 'Sharing',
        'action'      : 'string:${object_url}/sharing',
        'permissions' : (permissions.ManageProperties,),
         },
        {
        'id'          : 'view',
        'name'        : 'View',
        'action'      : 'string:${folder_url}/',
        'permissions' : (permissions.View,),
         },
        )
    )
    
    def canSelectDefaultPage(self):
        return False

    security.declarePrivate('validate_availableCategories')
    def validate_availableCategories(self, value):
        """Ensure that when setting available categories, we don't accidentally
        set the same category id as an existing project.
        """
        catalog = getToolByName(self, 'portal_catalog')
        projects = catalog.searchResults(
                            portal_type = 'PSCProject',
                            path = '/'.join(self.getPhysicalPath()))
        
        categoryIds = [v.split('|')[0].strip() for v in value]
        projectIds = [p.getId for p in projects]

        empty = []
        invalid = []
        for cat in categoryIds:
            if not cat:
                empty.append(cat)
            elif cat in projectIds or categoryIds.count(cat) > 1:
                invalid.append(cat)
    
        if empty:
            return "Please enter categories as Short Name | Long name | Description."
        if invalid:
            return "The following short category names are in use, either " \
                   "by another category, or by a project in the software " \
                   "center: %s." % (','.join(invalid))
        else:
            return None
            
    security.declarePrivate('_getContained')
    def _getContained(self, states, category, portal_type, limit=None, 
                        sort_on='Date', sort_order='reverse'):
        """Get contained objects of type portal_type
        that are in states and have category."""
        
        catalog = getToolByName(self, 'portal_catalog')
        my_path = '/'.join(self.getPhysicalPath())
        query = { 'path'         : my_path,
                  'portal_type'  : portal_type,
                }
                
        if states:
            query['review_state'] = states
        if category:
            query['getCategories'] = category
        if limit:
            query['sort_limit'] = limit
        if sort_on:
            query['sort_on'] = sort_on
        if sort_order:
            query['sort_order'] = sort_order
            
        results = catalog.searchResults(query)
        
        if limit:
            return results[:int(limit)]
        else:
            return results

    security.declareProtected(permissions.View, 'getActiveProjects')
    def getActiveProjects(self, states=[], limit=None):
        """Get all active projects (i.e. they have one alpha/beta/rc/final 
        release).
        """
        catalog = getToolByName(self, 'portal_catalog')
        return catalog(review_state = 'published',
                       portal_type = 'PSCProject',
                       path = '/'.join(self.getPhysicalPath()),
                       releaseCount = {'query' : 1, 'range' : 'min'},
                       sort_on = 'sortable_title',
                       sort_order = 'asc')
        
    security.declareProtected(permissions.View, 'getProjectsByCategory')
    def getProjectsByCategory(self, category, states=[], limit=None):
        """Get catalog brains for projects in category."""
        return self._getContained(states, category, 'PSCProject', limit,
                                    sort_on='sortable_title', sort_order = 'asc')

    security.declareProtected(permissions.View, 'getReleases')
    def getReleases(self, states=[], limit=None):
        """Get catalog brain of releases."""
        return self._getContained(states, None, 'PSCRelease', limit)

    security.declareProtected(permissions.View, 'getReleasesByCategory')
    def getReleasesByCategory(self, category, states=['alpha', 'beta', 'release-candiate', 'final'], limit=None):
        """Get catalog brains for releases in category."""
        return self._getContained(states, category, 'PSCRelease', limit, sort_on='Date', sort_order='reverse')

    security.declareProtected(permissions.View, 'getCategoriesToList') 
    def getCategoriesToList(self, states=[]): 
        """Get a list of category ids that have at least one listable package""" 
        vocab = self.getAvailableCategoriesAsDisplayList() 
        catalog = getToolByName(self, 'portal_catalog') 
        uniqueCategories = catalog.uniqueValuesFor('getCategories') 

        categories = [] 
        for cat in vocab.keys(): 
            if cat in uniqueCategories: 
                categories.append(cat) 

        return categories

    security.declareProtected(permissions.View, 'getCategoryName')
    def getCategoryName(self, category):
        """Get the long name of a category.
        """
        return self.getField('availableCategories').lookup(self, category, 1)

    security.declareProtected(permissions.View, 'getCategoryDescription')
    def getCategoryDescription(self, category):
        """Get the description of a category.
        """
        return self.getField('availableCategories').lookup(self, category, 2)

    security.declareProtected(permissions.View,
                              'getAvailableCategoriesAsDisplayList')
    def getAvailableCategoriesAsDisplayList(self):
        """Get categories in DisplayList form."""
        return self.getField('availableCategories').getAsDisplayList(self)

    security.declareProtected(permissions.View,
                              'getAvailableLicensesAsDisplayList')
    def getAvailableLicensesAsDisplayList(self):
        """Get licenses in DisplayList form."""
        return self.getField('availableLicenses').getAsDisplayList(self)

    security.declareProtected(permissions.View,
                              'getAvailableMaturitiesAsDisplayList')
    def getAvailableMaturitiesAsDisplayList(self):
        """Get licenses in DisplayList form."""
        return self.getField('availableMaturities').getAsDisplayList(self)

    security.declareProtected(permissions.View,
                              'getAvailableVersionsAsDisplayList')
    def getAvailableVersionsAsDisplayList(self):
        return DisplayList([(item, item) for item in self.getAvailableVersions()])


registerType(PloneSoftwareCenter, PROJECTNAME)
